﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace GrafClassLibrary
{
    class MatrixGraf
    {
        private int[,] matrix;
        private int antal;

        private void initMatrix()
        {
            for (int fra = 0; fra < antal; ++fra)
            {
                for (int til = 0; til < antal; ++til)
                {
                    matrix[fra, til] = -1;
                }

            }
        }
        public void AddKantUndirected(int fra, int til, int vægt)
        {
            matrix[fra, til] = vægt;
            matrix[til, fra] = vægt;
        }


        public class Kant : IComparable<Kant>
        {
            public int Fra;
            public int Til;
            public int Vægt;
            public Kant(int fra, int til, int vægt)
            {
                this.Fra = fra;
                this.Til = til;
                this.Vægt = vægt;
            }

            public int CompareTo(Kant other)
            {
                return this.Vægt - other.Vægt;
            }
        }

        public List<Kant> Kruskal()
        {
            List<Kant> arbejdsKanter = new List<Kant>();
            // opbyg liste med alle kanter
            for (int fra = 0; fra < antal; ++fra)
            {
                //for (int til = 0; til < antal; ++til)
                for (int til = 0; til <= fra; )     // optimering af kanter
                {
                    if (matrix[fra, til] != -1) // der er en kant
                    {
                        Kant k = new Kant(fra, til, matrix[fra, til]);
                        arbejdsKanter.Add(k);
                    }
                }
            }
            arbejdsKanter.Sort();
            List<Kant> valgteKanter = new List<Kant>();


            int[] grafnumre = new int[antal];
            int sidstegrafnummer = 0;
            foreach (Kant k in arbejdsKanter)
            {
                bool skalmedtages = true;
                // find ud af om den skal med
                // hvis vi har sørget for at vi har en graf med de hidtil fundne kanter
                // kunne vi nu bruge dybbde-/bredde søgning med udgangspunkt
                // i fra-punkt og se om vi kunne komme til til-punkt
                // for at afgøre om kant skulle med - den må ikke være cyklisk 
                if (grafnumre[k.Til] == 0 && grafnumre[k.Fra] == 0) // ingen af de to er med endnu
                {
                    //skalmedtages = true;
                    ++sidstegrafnummer;
                    grafnumre[k.Til] = sidstegrafnummer;
                    grafnumre[k.Fra] = sidstegrafnummer;
                }
                else if (grafnumre[k.Til] == 0) // til er ikke "besøgt"/med endnu 
                {
                    //skalmedtages = true;
                    grafnumre[k.Til] = grafnumre[k.Fra];   // samme graf som fra tilhører
                }
                else if (grafnumre[k.Fra] == 0) // fra er ikke "besøgt"/med endnu 
                {
                    //skalmedtages = true;
                    grafnumre[k.Fra] = grafnumre[k.Til];   // samme graf som til tilhører
                }
                else // begge er med i en graf / er "besøgt"
                {
                    if (grafnumre[k.Til] == grafnumre[k.Fra])
                        skalmedtages = false;
                    else
                    {
                        //    skalmedtages = true;
                        for (int i = 0; i < antal; ++i)     // check alle knuder
                        {
                            if (grafnumre[i] == grafnumre[k.Til])   // hvis med i "graf-til"
                                grafnumre[i] = grafnumre[k.Fra];    // ændres til "graf-fra" så de er i samme graf
                        }
                    }
                }
                if (skalmedtages)
                {
                    valgteKanter.Add(k);
                }
            }
            return valgteKanter;
        }

        public List<Kant> Prims(int start)
        {
            List<Kant> valgteKanter = new List<Kant>();

            bool[] medtaget = new bool[antal];
            medtaget[start] = true;
            int antalMedtaget = 1;
            while (antalMedtaget < antal)
            {
                // find næste kant der skal med
                Kant k = new Kant(-1, -1, int.MaxValue);
                for (int fra = 0; fra < antal; ++fra)
                {
                    // Nye muligheder
                    if (medtaget[fra])
                    {
                        for (int til = 0; til < antal; ++til) 
                        {
                            if (matrix[fra, til] != 1)// se på alle muligheder
                            {
                                if (!medtaget[til]) // kun nye dur
                                {
                                    if (matrix[fra, til] < k.Vægt) // denne er billigere
                                    {
                                        k.Fra = fra;
                                        k.Til = til;
                                        k.Vægt = matrix[fra, til];
                                    }
                                }
                            }
                        }
                    }
                }
                if (k.Til == -1) // må ikke være stadig fiktiv ~ ingen fundet - usammenhængende
                    break;
                // ok - nye skal med
                valgteKanter.Add(k);
                medtaget[k.Til] = true;
            }
            return valgteKanter;
        }

        public List<Kant> ShortestPath(int start)  // lidt naiv Dijkstra
        {
            List<Kant> valgteKanter = new List<Kant>();

            bool[] medtaget = new bool[antal];
            int[] samletVægt = new int[antal];      // vægt fra start akkumuleret
            medtaget[start] = true;
            int antalMedtaget = 1;
            while (antalMedtaget < antal)
            {
                // find næste kant der skal med
                Kant k = new Kant(-1, -1, int.MaxValue);
                for (int fra = 0; fra < antal; ++fra)
                {
                    // Nye muligheder
                    if (medtaget[fra])
                    {
                        for (int til = 0; til < antal; ++til)
                        {
                            if (matrix[fra, til] != 1)// se på alle muligheder
                            {
                                if (!medtaget[til]) // kun nye dur
                                {
                                    // HER SKAL VI JUSTERE
                                    int sum = samletVægt[fra] + matrix[fra, til];
                                    if (sum < k.Vægt) // denne er billigere
                                    {
                                        k.Fra = fra;
                                        k.Til = til;
                                        k.Vægt = sum;  // samlet vægt fra start
                                    }
                                }
                            }
                        }
                    }
                }
                if (k.Til == -1) // må ikke være stadig fiktiv ~ ingen fundet - usammenhængende
                    break;
                // ok - nye skal med
                valgteKanter.Add(k);
                medtaget[k.Til] = true;
                samletVægt[k.Til] = k.Vægt;         // husk samlet vægt
            }
            return valgteKanter;
        }
    
    }
}
